@@ -1,8 +1,8 @@
 /*****************************************************************************
- * xml.h
+ * xml.h: XML abstraction layer
  *****************************************************************************
  * Copyright (C) 2004 the VideoLAN team
- * $Id: vlc_xml.h 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id$
  *
  * Author: Gildas Bazin <gbazin@videolan.org>
  *
@@ -18,11 +18,17 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
-#ifndef _VLC_XML_H
-#define _VLC_XML_H
+#ifndef VLC_XML_H
+#define VLC_XML_H
+
+/**
+ * \file
+ * This file defines functions and structures to handle xml tags in vlc
+ *
+ */
 
 # ifdef __cplusplus
 extern "C" {
@@ -64,7 +70,7 @@ struct xml_reader_t
     char * (*pf_value) ( xml_reader_t * );
     int (*pf_next_attr) ( xml_reader_t * );
 
-    int (*pf_use_dtd) ( xml_reader_t *, vlc_bool_t );
+    int (*pf_use_dtd) ( xml_reader_t *, bool );
 };
 
 #define xml_ReaderRead( a ) a->pf_read( a )
