@@ -2,7 +2,7 @@
  * http.c
  *****************************************************************************
  * Copyright (C) 2001-2005 the VideoLAN team
- * $Id: http.c 17020 2006-10-10 18:33:26Z fkuehne $
+ * $Id: 8b194fce95801ea6e0b6d19c3bf5cfd21e6dd8e9 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Jon Lech Johansen <jon@nanocrew.net>
@@ -25,14 +25,21 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdlib.h>
 
-#include <vlc/vlc.h>
-#include <vlc/sout.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_plugin.h>
+#include <vlc_sout.h>
+#include <vlc_block.h>
+
+
+#include <vlc_input.h>
+#include <vlc_playlist.h>
 
 #ifdef HAVE_AVAHI_CLIENT
-    #include <vlc/intf.h>
-
     #include "bonjour.h"
 
     #if defined( WIN32 )
@@ -44,8 +51,6 @@
 
 #include "vlc_httpd.h"
 
-#define FREE( p ) if( p ) { free( p); (p) = NULL; }
-
 #define DEFAULT_PORT        8080
 #define DEFAULT_SSL_PORT    8443
 
@@ -63,12 +68,9 @@ static void Close( vlc_object_t * );
 #define PASS_TEXT N_("Password")
 #define PASS_LONGTEXT N_("Password that will be " \
                          "requested to access the stream." )
-
-/// \bug [String] missing closing parenthesis
 #define MIME_TEXT N_("Mime")
 #define MIME_LONGTEXT N_("MIME returned by the server (autodetected " \
-                        "if not specified." )
-
+                        "if not specified)." )
 #define CERT_TEXT N_( "Certificate file" )
 #define CERT_LONGTEXT N_( "Path to the x509 PEM certificate file that will "\
                           "be used for HTTPS." )
@@ -90,7 +92,7 @@ static void Close( vlc_object_t * );
 
 
 vlc_module_begin();
-    set_description( _("HTTP stream output") );
+    set_description( N_("HTTP stream output") );
     set_capability( "sout access", 0 );
     set_shortname( "HTTP" );
     add_shortcut( "http" );
@@ -99,21 +101,21 @@ vlc_module_begin();
     set_category( CAT_SOUT );
     set_subcategory( SUBCAT_SOUT_ACO );
     add_string( SOUT_CFG_PREFIX "user", "", NULL,
-                USER_TEXT, USER_LONGTEXT, VLC_TRUE );
+                USER_TEXT, USER_LONGTEXT, true );
     add_string( SOUT_CFG_PREFIX "pwd", "", NULL,
-                PASS_TEXT, PASS_LONGTEXT, VLC_TRUE );
+                PASS_TEXT, PASS_LONGTEXT, true );
     add_string( SOUT_CFG_PREFIX "mime", "", NULL,
-                MIME_TEXT, MIME_LONGTEXT, VLC_TRUE );
+                MIME_TEXT, MIME_LONGTEXT, true );
     add_string( SOUT_CFG_PREFIX "cert", "vlc.pem", NULL,
-                CERT_TEXT, CERT_LONGTEXT, VLC_TRUE );
+                CERT_TEXT, CERT_LONGTEXT, true );
     add_string( SOUT_CFG_PREFIX "key", NULL, NULL,
-                KEY_TEXT, KEY_LONGTEXT, VLC_TRUE );
+                KEY_TEXT, KEY_LONGTEXT, true );
     add_string( SOUT_CFG_PREFIX "ca", NULL, NULL,
-                CA_TEXT, CA_LONGTEXT, VLC_TRUE );
+                CA_TEXT, CA_LONGTEXT, true );
     add_string( SOUT_CFG_PREFIX "crl", NULL, NULL,
-                CRL_TEXT, CRL_LONGTEXT, VLC_TRUE );
-    add_bool( SOUT_CFG_PREFIX "bonjour", VLC_FALSE, NULL,
-              BONJOUR_TEXT, BONJOUR_LONGTEXT, VLC_TRUE);
+                CRL_TEXT, CRL_LONGTEXT, true );
+    add_bool( SOUT_CFG_PREFIX "bonjour", false, NULL,
+              BONJOUR_TEXT, BONJOUR_LONGTEXT, true);
     set_callbacks( Open, Close );
 vlc_module_end();
 
@@ -121,11 +123,11 @@ vlc_module_end();
 /*****************************************************************************
  * Exported prototypes
  *****************************************************************************/
-static const char *ppsz_sout_options[] = {
+static const char *const ppsz_sout_options[] = {
     "user", "pwd", "mime", "cert", "key", "ca", "crl", NULL
 };
 
-static int Write( sout_access_out_t *, block_t * );
+static ssize_t Write( sout_access_out_t *, block_t * );
 static int Seek ( sout_access_out_t *, off_t  );
 
 struct sout_access_out_sys_t
@@ -140,7 +142,7 @@ struct sout_access_out_sys_t
     int                 i_header_allocated;
     int                 i_header_size;
     uint8_t             *p_header;
-    vlc_bool_t          b_header_complete;
+    bool          b_header_complete;
 
 #ifdef HAVE_AVAHI_CLIENT
     void                *p_bonjour;
@@ -163,62 +165,50 @@ static int Open( vlc_object_t *p_this )
     char                *psz_user = NULL;
     char                *psz_pwd = NULL;
     char                *psz_mime = NULL;
-    const char          *psz_cert = NULL, *psz_key = NULL, *psz_ca = NULL,
+    char                *psz_cert = NULL, *psz_key = NULL, *psz_ca = NULL,
                         *psz_crl = NULL;
     vlc_value_t         val;
 
     if( !( p_sys = p_access->p_sys =
                 malloc( sizeof( sout_access_out_sys_t ) ) ) )
-    {
-        msg_Err( p_access, "Not enough memory" );
         return VLC_ENOMEM ;
-    }
 
-    sout_CfgParse( p_access, SOUT_CFG_PREFIX, ppsz_sout_options, p_access->p_cfg );
+    config_ChainParse( p_access, SOUT_CFG_PREFIX, ppsz_sout_options, p_access->p_cfg );
 
-    /* p_access->psz_name = "hostname:port/filename" */
-    psz_bind_addr = psz_parser = strdup( p_access->psz_name );
+    /* p_access->psz_path = "hostname:port/filename" */
+    psz_bind_addr = psz_parser = strdup( p_access->psz_path );
 
     i_bind_port = 0;
-    psz_file_name = "";
 
-    while( *psz_parser && *psz_parser != ':' && *psz_parser != '/' )
-    {
-        psz_parser++;
-    }
-    if( *psz_parser == ':' )
+    psz_parser = strchr( psz_bind_addr, '/' );
+    if( psz_parser )
     {
+        psz_file_name = strdup( psz_parser );
         *psz_parser = '\0';
-        psz_parser++;
-        i_bind_port = atoi( psz_parser );
-
-        while( *psz_parser && *psz_parser != '/' )
-        {
-            psz_parser++;
-        }
     }
-    if( *psz_parser == '/' )
-    {
-        *psz_parser = '\0';
-        psz_parser++;
-        psz_file_name = psz_parser;
-    }
-
-    if( !*psz_file_name )
-    {
+    else
         psz_file_name = strdup( "/" );
-    }
-    else if( *psz_file_name != '/' )
-    {
-        char *p = psz_file_name;
 
-        psz_file_name = malloc( strlen( p ) + 2 );
-        strcpy( psz_file_name, "/" );
-        strcat( psz_file_name, p );
+    if( psz_bind_addr[0] == '[' )
+    {
+        psz_bind_addr++;
+        psz_parser = strstr( psz_bind_addr, "]:" );
+        if( psz_parser )
+        {
+            *psz_parser = '\0';
+            i_bind_port = atoi( psz_parser + 2 );
+        }
+        psz_parser = psz_bind_addr - 1;
     }
     else
     {
-        psz_file_name = strdup( psz_file_name );
+        psz_parser = strrchr( psz_bind_addr, ':' );
+        if( psz_parser )
+        {
+            *psz_parser = '\0';
+            i_bind_port = atoi( psz_parser + 1 );
+        }
+        psz_parser = psz_bind_addr;
     }
 
     /* SSL support */
@@ -242,16 +232,21 @@ static int Open( vlc_object_t *p_this )
                                             psz_bind_addr, i_bind_port,
                                             psz_cert, psz_key, psz_ca,
                                             psz_crl );
+    free( psz_cert );
+    free( psz_key );
+    free( psz_ca );
+    free( psz_crl );
+
     if( p_sys->p_httpd_host == NULL )
     {
-        msg_Err( p_access, "cannot listen on %s:%d",
+        msg_Err( p_access, "cannot listen on %s port %d",
                  psz_bind_addr, i_bind_port );
         free( psz_file_name );
-        free( psz_bind_addr );
+        free( psz_parser );
         free( p_sys );
         return VLC_EGENERIC;
     }
-    free( psz_bind_addr );
+    free( psz_parser );
 
     if( p_access->psz_access && !strcmp( p_access->psz_access, "mmsh" ) )
     {
@@ -281,9 +276,9 @@ static int Open( vlc_object_t *p_this )
     p_sys->p_httpd_stream =
         httpd_StreamNew( p_sys->p_httpd_host, psz_file_name, psz_mime,
                          psz_user, psz_pwd, NULL );
-    if( psz_user ) free( psz_user );
-    if( psz_pwd ) free( psz_pwd );
-    if( psz_mime ) free( psz_mime );
+    free( psz_user );
+    free( psz_pwd );
+    free( psz_mime );
 
     if( p_sys->p_httpd_stream == NULL )
     {
@@ -298,39 +293,33 @@ static int Open( vlc_object_t *p_this )
 #ifdef HAVE_AVAHI_CLIENT
     if( config_GetInt(p_this, SOUT_CFG_PREFIX "bonjour") )
     {
-        playlist_t          *p_playlist;
         char                *psz_txt, *psz_name;
+        playlist_t          *p_playlist = pl_Yield( p_access );
 
-        p_playlist = (playlist_t *)vlc_object_find( p_access,
-                                                    VLC_OBJECT_PLAYLIST,
-                                                    FIND_ANYWHERE );
-        if( p_playlist == NULL )
-        {
-            msg_Err( p_access, "unable to find playlist" );
-            httpd_StreamDelete( p_sys->p_httpd_stream );
-            httpd_HostDelete( p_sys->p_httpd_host );
-            free( (void *)p_sys );
-            return VLC_EGENERIC;
-        }
-
-        psz_name = strrchr( p_playlist->status.p_item->input.psz_uri,
-                            DIRECTORY_SEPARATOR );
+        char *psz_uri = input_item_GetURI( p_playlist->status.p_item->p_input );
+        char *psz_newuri = psz_uri;
+        psz_name = strrchr( psz_newuri, DIRECTORY_SEPARATOR );
         if( psz_name != NULL ) psz_name++;
-        else psz_name = p_playlist->status.p_item->input.psz_uri;
+        else psz_name = psz_newuri;
 
-        asprintf( &psz_txt, "path=%s", psz_file_name );
+        if( psz_file_name &&
+            asprintf( &psz_txt, "path=%s", psz_file_name ) == -1 )
+            {
+                pl_Release( p_access );
+                free( psz_uri );
+                return VLC_ENOMEM;
+            }
 
         p_sys->p_bonjour = bonjour_start_service( (vlc_object_t *)p_access,
                                     strcmp( p_access->psz_access, "https" )
                                        ? "_vlc-http._tcp" : "_vlc-https._tcp",
-                                                  psz_name, i_bind_port, psz_txt );
-        free( (void *)psz_txt );
+                                             psz_name, i_bind_port, psz_txt );
+        free( psz_uri );
+        free( psz_txt );
 
         if( p_sys->p_bonjour == NULL )
-        {
-            msg_Err( p_access, "Avahi stream announcing was requested, but no avahi service could be started" );
-        }
-        vlc_object_release( p_playlist );
+            msg_Err( p_access, "unable to start requested Bonjour announce" );
+        pl_Release( p_access );
     }
     else
         p_sys->p_bonjour = NULL;
@@ -341,7 +330,7 @@ static int Open( vlc_object_t *p_this )
     p_sys->i_header_allocated = 1024;
     p_sys->i_header_size      = 0;
     p_sys->p_header           = malloc( p_sys->i_header_allocated );
-    p_sys->b_header_complete  = VLC_FALSE;
+    p_sys->b_header_complete  = false;
 
     p_access->pf_write       = Write;
     p_access->pf_seek        = Seek;
@@ -372,7 +361,7 @@ static void Close( vlc_object_t * p_this )
     httpd_StreamDelete( p_sys->p_httpd_stream );
     httpd_HostDelete( p_sys->p_httpd_host );
 
-    FREE( p_sys->p_header );
+    free( p_sys->p_header );
 
     msg_Dbg( p_access, "Close" );
 
@@ -382,10 +371,11 @@ static void Close( vlc_object_t * p_this )
 /*****************************************************************************
  * Write:
  *****************************************************************************/
-static int Write( sout_access_out_t *p_access, block_t *p_buffer )
+static ssize_t Write( sout_access_out_t *p_access, block_t *p_buffer )
 {
     sout_access_out_sys_t *p_sys = p_access->p_sys;
     int i_err = 0;
+    int i_len = 0;
 
     while( p_buffer )
     {
@@ -398,7 +388,7 @@ static int Write( sout_access_out_t *p_access, block_t *p_buffer )
             {
                 /* free previously gathered header */
                 p_sys->i_header_size = 0;
-                p_sys->b_header_complete = VLC_FALSE;
+                p_sys->b_header_complete = false;
             }
             if( (int)(p_buffer->i_buffer + p_sys->i_header_size) >
                 p_sys->i_header_allocated )
@@ -415,12 +405,13 @@ static int Write( sout_access_out_t *p_access, block_t *p_buffer )
         }
         else if( !p_sys->b_header_complete )
         {
-            p_sys->b_header_complete = VLC_TRUE;
+            p_sys->b_header_complete = true;
 
             httpd_StreamHeader( p_sys->p_httpd_stream, p_sys->p_header,
                                 p_sys->i_header_size );
         }
 
+        i_len += p_buffer->i_buffer;
         /* send data */
         i_err = httpd_StreamSend( p_sys->p_httpd_stream, p_buffer->p_buffer,
                                   p_buffer->i_buffer );
@@ -440,7 +431,7 @@ static int Write( sout_access_out_t *p_access, block_t *p_buffer )
         block_ChainRelease( p_buffer );
     }
 
-    return( i_err < 0 ? VLC_EGENERIC : VLC_SUCCESS );
+    return( i_err < 0 ? VLC_EGENERIC : i_len );
 }
 
 /*****************************************************************************
@@ -448,6 +439,7 @@ static int Write( sout_access_out_t *p_access, block_t *p_buffer )
  *****************************************************************************/
 static int Seek( sout_access_out_t *p_access, off_t i_pos )
 {
+    (void)i_pos;
     msg_Warn( p_access, "HTTP sout access cannot seek" );
     return VLC_EGENERIC;
 }
