@@ -2,7 +2,7 @@
  * float32.c : precise float32 audio mixer implementation
  *****************************************************************************
  * Copyright (C) 2002 the VideoLAN team
- * $Id: float32.c 13905 2006-01-12 23:10:04Z dionoea $
+ * $Id$
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -10,7 +10,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -24,12 +24,15 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdlib.h>                                      /* malloc(), free() */
-#include <string.h>
 
-#include <vlc/vlc.h>
-#include "audio_output.h"
-#include "aout_internal.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <stddef.h>
+#include <vlc_common.h>
+#include <vlc_plugin.h>
+#include <vlc_aout.h>
 
 /*****************************************************************************
  * Local prototypes
@@ -44,7 +47,7 @@ static void DoWork    ( aout_instance_t *, aout_buffer_t * );
 vlc_module_begin();
     set_category( CAT_AUDIO );
     set_subcategory( SUBCAT_AUDIO_MISC );
-    set_description( _("Float32 audio mixer") );
+    set_description( N_("Float32 audio mixer") );
     set_capability( "audio mixer", 10 );
     set_callbacks( Create, NULL );
 vlc_module_end();
@@ -61,14 +64,20 @@ static int Create( vlc_object_t *p_this )
         return -1;
     }
 
+    /* Use the trivial mixer when we can */
     if ( p_aout->i_nb_inputs == 1 && p_aout->mixer.f_multiplier == 1.0 )
     {
-        /* Tell the trivial mixer to go for it. */
-        return -1;
+        int i;
+        for( i = 0; i < p_aout->i_nb_inputs; i++ )
+        {
+            if( p_aout->pp_inputs[i]->f_multiplier != 1.0 )
+                break;
+        }
+        if( i >= p_aout->i_nb_inputs )
+            return -1;
     }
 
     p_aout->mixer.pf_do_work = DoWork;
-
     return 0;
 }
 
@@ -110,15 +119,17 @@ static void MeanWords( float * p_out, const float * p_in, size_t i_nb_words,
  *****************************************************************************/
 static void DoWork( aout_instance_t * p_aout, aout_buffer_t * p_buffer )
 {
-    int i_nb_inputs = p_aout->i_nb_inputs;
-    float f_multiplier = p_aout->mixer.f_multiplier;
+    const int i_nb_inputs = p_aout->i_nb_inputs;
+    const float f_multiplier_global = p_aout->mixer.f_multiplier;
+    const int i_nb_channels = aout_FormatNbChannels( &p_aout->mixer.mixer );
     int i_input;
-    int i_nb_channels = aout_FormatNbChannels( &p_aout->mixer.mixer );
 
     for ( i_input = 0; i_input < i_nb_inputs; i_input++ )
     {
         int i_nb_words = p_buffer->i_nb_samples * i_nb_channels;
         aout_input_t * p_input = p_aout->pp_inputs[i_input];
+        float f_multiplier = f_multiplier_global * p_input->f_multiplier;
+
         float * p_out = (float *)p_buffer->p_buffer;
         float * p_in = (float *)p_input->p_first_byte_to_mix;
 
