@@ -1,16 +1,16 @@
 /*****************************************************************************
  * spudec.h : sub picture unit decoder thread interface
  *****************************************************************************
- * Copyright (C) 1999, 2000 the VideoLAN team
- * $Id: spudec.h 11664 2005-07-09 06:17:09Z courmisch $
+ * Copyright (C) 1999, 2000, 2006 the VideoLAN team
+ * $Id$
  *
- * Authors: Samuel Hocevar <sam@zoy.org>
+ * Authors: Sam Hocevar <sam@zoy.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -18,22 +18,25 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
+/* #define DEBUG_SPUDEC 1 */
+
 struct decoder_sys_t
 {
     int b_packetizer;
 
     mtime_t i_pts;
-    int i_spu_size;
-    int i_rle_size;
-    int i_spu;
+    int     i_rate;
+    unsigned int i_spu_size;
+    unsigned int i_rle_size;
+    unsigned int i_spu;
 
     block_t *p_block;
 
-    /* We will never overflow more than 11 bytes if I'm right */
-    uint8_t buffer[65536 + 20 ];
+    /* We will never overflow */
+    uint8_t buffer[65536];
 };
 
 typedef struct subpicture_data_t
@@ -44,12 +47,12 @@ typedef struct subpicture_data_t
     void *p_data;
 
     /* Color information */
-    vlc_bool_t b_palette;
+    bool b_palette;
     uint8_t    pi_alpha[4];
     uint8_t    pi_yuv[4][3];
 
     /* Auto crop fullscreen subtitles */
-    vlc_bool_t b_auto_crop;
+    bool b_auto_crop;
     int i_y_top_offset;
     int i_y_bottom_offset;
 
@@ -75,4 +78,4 @@ typedef struct subpicture_data_t
 /*****************************************************************************
  * Prototypes
  *****************************************************************************/
-subpicture_t * E_(ParsePacket)( decoder_t * );
+subpicture_t * ParsePacket( decoder_t * );
