@@ -1,8 +1,8 @@
 /*****************************************************************************
  * intf_beos.cpp: beos interface
  *****************************************************************************
- * Copyright (C) 1999, 2000, 2001 VideoLAN
- * $Id: Interface.cpp,v 1.11 2003/03/12 23:15:03 titer Exp $
+ * Copyright (C) 1999, 2000, 2001 the VideoLAN team
+ * $Id: 859c618e8a1d26374dd886338545c0f4e14e5b31 $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -23,28 +23,36 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdio.h>
-#include <stdlib.h>                                      /* malloc(), free() */
 #include <InterfaceKit.h>
 #include <Application.h>
 #include <Message.h>
-#include <string.h>
 
-#include <vlc/vlc.h>
-#include <vlc/intf.h>
-#include <vlc/aout.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_interface.h>
+#include <vlc_aout.h>
 #include <aout_internal.h>
 
-#include "VlcWrapper.h"
 #include "InterfaceWindow.h"
 #include "MsgVals.h"
 
+/*****************************************************************************
+ * intf_sys_t: internal variables of the BeOS interface
+ *****************************************************************************/
+struct intf_sys_t
+{
+    InterfaceWindow * p_window;
+};
+
 /*****************************************************************************
  * Local prototype
  *****************************************************************************/
@@ -53,65 +61,56 @@ static void Run       ( intf_thread_t *p_intf );
 /*****************************************************************************
  * intf_Open: initialize interface
  *****************************************************************************/
-int E_(OpenIntf) ( vlc_object_t *p_this )
+int OpenIntf ( vlc_object_t *p_this )
 {
     intf_thread_t * p_intf = (intf_thread_t*) p_this;
 
     /* Allocate instance and initialize some members */
     p_intf->p_sys = (intf_sys_t*) malloc( sizeof( intf_sys_t ) );
-    if( p_intf->p_sys == NULL )
+    if( !p_intf->p_sys )
     {
         msg_Err( p_intf, "out of memory" );
-        return( 1 );
+        return VLC_EGENERIC;
     }
-    
-    p_intf->p_sys->p_sub = msg_Subscribe( p_intf );
-    p_intf->p_sys->p_wrapper = new VlcWrapper( p_intf );
+ 
     p_intf->pf_run = Run;
 
     /* Create the interface window */
-    BScreen screen(B_MAIN_SCREEN_ID);
-    BRect rect = screen.Frame();
-    rect.top = rect.bottom-100;
+    BScreen screen( B_MAIN_SCREEN_ID );
+    BRect rect   = screen.Frame();
+    rect.top     = rect.bottom - 100;
     rect.bottom -= 50;
-    rect.left += 50;
-    rect.right = rect.left + 350;
+    rect.left   += 50;
+    rect.right   = rect.left + 350;
     p_intf->p_sys->p_window =
-        new InterfaceWindow( rect,
-                             "VLC " PACKAGE_VERSION,
-                             p_intf );
-    if( p_intf->p_sys->p_window == 0 )
+        new InterfaceWindow( p_intf, rect, "VLC " VERSION );
+    if( !p_intf->p_sys->p_window )
     {
         free( p_intf->p_sys );
         msg_Err( p_intf, "cannot allocate InterfaceWindow" );
-        return( 1 );
-    } else {
-        BMessage message(INTERFACE_CREATED);
-        message.AddPointer("window", p_intf->p_sys->p_window);
-        be_app->PostMessage(&message);
+        return VLC_EGENERIC;
     }
-    p_intf->p_sys->i_saved_volume = AOUT_VOLUME_DEFAULT;
-    p_intf->p_sys->b_loop = 0;
-    p_intf->p_sys->b_mute = 0;
-    
-    return( 0 );
+
+    /* Make the be_app aware the interface has been created */
+    BMessage message( INTERFACE_CREATED );
+    message.AddPointer( "window", p_intf->p_sys->p_window );
+    be_app->PostMessage( &message );
+
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
  * intf_Close: destroy dummy interface
  *****************************************************************************/
-void E_(CloseIntf) ( vlc_object_t *p_this )
+void CloseIntf ( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t*) p_this;
-    
-    msg_Unsubscribe( p_intf, p_intf->p_sys->p_sub );
 
     /* Destroy the interface window */
-    p_intf->p_sys->p_window->Lock();
-    p_intf->p_sys->p_window->Quit();
+    if( p_intf->p_sys->p_window->Lock() )
+        p_intf->p_sys->p_window->Quit();
 
     /* Destroy structure */
-    delete p_intf->p_sys->p_wrapper;
     free( p_intf->p_sys );
 }
 
@@ -121,15 +120,9 @@ void E_(CloseIntf) ( vlc_object_t *p_this )
  *****************************************************************************/
 static void Run( intf_thread_t *p_intf )
 {
-    while( !p_intf->b_die )
+    while( !intf_ShouldDie( p_intf ) )
     {
-        if( p_intf->p_sys->p_wrapper->UpdateInput() )
-        {
-            /* Manage the slider */
-            p_intf->p_sys->p_window->UpdateInterface();
-        }
-
-        /* Wait a bit */
+        p_intf->p_sys->p_window->UpdateInterface();
         msleep( INTF_IDLE_SLEEP );
     }
 }
