@@ -1,16 +1,16 @@
 /*****************************************************************************
  * equalizer.h: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2004 VideoLAN
- * $Id: equalizer.h 10101 2005-03-02 16:47:31Z robux4 $
+ * Copyright (C) 2004-2008 the VideoLAN team
+ * $Id$
  *
- * Authors: Jrme Decoodt <djc@videolan.org>
+ * Authors: Jérôme Decoodt <djc@videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -18,11 +18,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
- * VLCEqualizer interface 
+ * VLCEqualizer interface
  *****************************************************************************/
 @interface VLCEqualizer : NSObject
 {
@@ -54,4 +54,10 @@
 - (IBAction)twopass:(id)sender;
 - (void)windowWillClose:(NSNotification *)aNotification;
 - (void)awakeFromNib;
+
+- (void)setValue: (float)value forSlider: (int)index;
+- (id)getSliderByIndex: (int)index;
+- (void)setBandSlidersValues: (float *)values;
+- (void)initBandSliders;
+
 @end
