@@ -2,7 +2,7 @@
  * MacOSX_loop.hpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: macosx_loop.hpp 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id$
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *
@@ -18,13 +18,14 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef MACOSX_LOOP_HPP
 #define MACOSX_LOOP_HPP
 
 #include "../src/os_loop.hpp"
+#include <Carbon/Carbon.h>
 
 class MacOSXDisplay;
 class GenericWindow;
@@ -45,15 +46,15 @@ class MacOSXLoop: public OSLoop
         /// Exit the main loop
         virtual void exit();
 
+        // Handle a window event
+        void registerWindow( GenericWindow &rGenWin, WindowRef win );
+
     private:
         // Private because it's a singleton
         MacOSXLoop( intf_thread_t *pIntf );
         virtual ~MacOSXLoop();
         // Flag set to exit the loop
         bool m_exit;
-
-        // Handle a window event
-        void handleWindowEvent( EventRef pEvent );
 };
 
 #endif
